
/****************************************************************************
** pc58nice.c
****************************************************************************
*/

#define EXTERN extern

/*------------------------
** Include files
**------------------------
*/
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <sys/ioctl.h>

#if SOLARIS
#include "pc58.h"
#endif

#if LINUX
#include <linux/pc58.h>
#endif

/*----------------------------------------------------------------------
**  pc58_inb_p() - allows user code to read IO ports
**----------------------------------------------------------------------
*/
int pc58_inb_p( int fd, unsigned short port )
{
   struct pc58parms_t pc58;
   pc58.l[0] = port;
   ioctl( fd, PC58Inb, &pc58 );
   return pc58.l[1];
}

/*----------------------------------------------------------------------
**  pc58_outb_p() - allows user code to write to IO ports
**----------------------------------------------------------------------
*/
void pc58_outb_p( int fd, char value, unsigned short port )
{
   struct pc58parms_t pc58;
   pc58.l[0] = port;
   pc58.l[1] = value;
   ioctl( fd, PC58Outb, &pc58 );
}

/*----------------------------------------------------------------------
**  a16_get_config() - get configuration information from driver
**  Input:
**     fd - file descriptor to pc58.
**  Output:
**     Rbase - returns io base via (int *).
**----------------------------------------------------------------------
*/
int pc58_getConfig( int fd, struct pc58_table_t *config )
{
   struct pc58parms_t pc58;

   pc58.p[0] = config;
   if( ioctl( fd, PC58GetConfig, &pc58 ) < 0 )
      return PC58_ERR_IOCTL;

   return pc58.ret;
}

/*----------------------------------------------------------------------
**  pc58_mutTest() -
**----------------------------------------------------------------------
*/
int pc58_mutTest( int fd, int csec, int id )
{
   struct pc58parms_t pc58;
   pc58.l[0] = csec;
   pc58.l[1] = id;
   ioctl( fd, PC58MutTest, &pc58 );
   return pc58.ret;
}

/*----------------------------------------------------------------------
**  pc58_puts() - writes the null terminated string s to the pc58.
**----------------------------------------------------------------------
*/
int pc58_puts( int fd, char *s )
{
   struct pc58parms_t pc58;
   pc58.p[0] = s;
   if( ioctl( fd, PC58Puts, &pc58 ) < 0 )
      return PC58_ERR_IOCTL;

   return pc58.ret;
}

/*----------------------------------------------------------------------
**  pc58_gets() - Reads the pc58 input buffer and write a null terminated
**                string to s.
**----------------------------------------------------------------------
*/
int pc58_gets( int fd, char *s, int s_size )
{
   struct pc58parms_t pc58;
   pc58.p[0] = s;
   pc58.l[0] = s_size;
   if( ioctl( fd, PC58Gets, &pc58 ) < 0 )
      return PC58_ERR_IOCTL;

   return pc58.ret;
}

/*----------------------------------------------------------------------
**  pc58_readAxisInfo() - Read the axis information from the 
**                         shared memory interface
**----------------------------------------------------------------------
*/
int pc58_readAxisInfo( int fd, int inx, struct pc58_dram_axis_t * dest_buf )
{
   struct pc58parms_t pc58;
   pc58.l[0] = inx;
   pc58.p[0] = dest_buf;
   if( ioctl( fd, PC58ReadAxisInfo, &pc58 ) < 0 )
      return PC58_ERR_IOCTL;

   return pc58.ret;
}

/*----------------------------------------------------------------------
**  pc58_command() - Sends a command and read a reply from the PC58.
**----------------------------------------------------------------------
*/
int pc58_command( int fd, char *command, char *reply, int r_size )
{
   struct pc58parms_t pc58;
   pc58.p[0] = command;
   pc58.p[1] = reply;
   pc58.l[0] = r_size;
   if( ioctl( fd, PC58Command, &pc58 ) < 0 )
      return PC58_ERR_IOCTL;

   return pc58.ret;
}

/*----------------------------------------------------------------------
**  pc58_clearSlipFlag() - Clears the slip flag variable in the driver.
**----------------------------------------------------------------------
*/
int pc58_clearSlipFlag( int fd, int mask )
{
   struct pc58parms_t pc58;
   pc58.l[0] = mask;
   if( ioctl( fd, PC58ClearSlipFlag, &pc58 ) < 0 )
      return PC58_ERR_IOCTL;

   return pc58.ret;
}

/*----------------------------------------------------------------------
**  pc58_clearDoneFlag() - Clears the done flag variable in the driver.
**----------------------------------------------------------------------
*/
int pc58_clearDoneFlag( int fd, int mask )
{
   struct pc58parms_t pc58;
   pc58.l[0] = mask;
   if( ioctl( fd, PC58ClearDoneFlag, &pc58 ) < 0 )
      return PC58_ERR_IOCTL;

   return pc58.ret;
}
/*----------------------------------------------------------------------
**  pc58_clearLimitFlag() - Clears the Limit flag variable in the driver.
**----------------------------------------------------------------------
*/
int pc58_clearLimitFlag( int fd, int mask )
{
   struct pc58parms_t pc58;
   pc58.l[0] = mask;
   if( ioctl( fd, PC58ClearLimitFlag, &pc58 ) < 0 )
      return PC58_ERR_IOCTL;

   return pc58.ret;
}

/*----------------------------------------------------------------------
**  pc58_clearHomeFlag() - Clears the slip flag variable in the driver.
**----------------------------------------------------------------------
*/
int pc58_clearHomeFlag( int fd, int mask )
{
   struct pc58parms_t pc58;
   pc58.l[0] = mask;
   if( ioctl( fd, PC58ClearHomeFlag, &pc58 ) < 0 )
      return PC58_ERR_IOCTL;

   return pc58.ret;
}


