
#define EXTERN
#define MAIN     1
 
/*--------------------------
**  include files
**--------------------------
*/
#include <sys/types.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include <fcntl.h>

#include <linux/pc58.h> 

#include "addlib.h"
#include "error_msg.h"

/*---------------------------
** function prototypes
**---------------------------
*/
int cmd_execute( char * cmd_buf);
int cmd_search_selection_list( char * item, char ** list);

int do_axisinfo( char *par );
int do_beep( char *par );
int do_do( char *par );
int do_cleardone( char *par );
int do_getconfig( char *par );
int do_gets( char *par );
int do_help( char *par );
int do_io( char *par );
int do_peek( char *par );
int do_poke( char *par );
int do_puts( char *par );
int do_quit( char *par );
int do_sleep( char *par );
int do_test( char *par );
int do_waitdone( char *par );

ulong get_ulong( char *prompt, ulong cur_val, ulong min, ulong max );
long get_long ( char *prompt, long cur_val, long min, long max );
double get_dbl( char *prompt, double cur_val, double min, double max );
 
/*-------------------------------------
*  Global variables
*-------------------------------------
*/
 
struct PARSE_TABLE
{
  char * kw;          /* The opcode string  */
  int (* pfi)();      /* pfi is pointer to function returning an integer */
  char * desc;        /* Description of command */
};
 
struct PARSE_TABLE parse_table[] =
{
	{ "axisinfo",   do_axisinfo,    "AXISINFO - Queries for axis info (interactive)." },
	{ "beep",       do_beep,        "BEEP - Outputs the bell characters" },
	{ "cleardone",  do_cleardone,   "CLEARDONE mask - clears the done flag." },
	{ "do",         do_do,          "DO filename - execute command from filename." },
   { "getconfig",  do_getconfig,   "GETCONFIG - display pc58 configuration info." },
	{ "gets",       do_gets,        "GETS - Reads the PC58 output buffer." },
	{ "help",       do_help,        "HELP - Prints this message." },
	{ "io",         do_io,          "IO - Interactive communicationn with the PC58." },
	{ "peek",       do_peek,        "PEEK - Read bytes to io ports (Interactive)." },
	{ "poke",       do_poke,        "POKE - Write bytes to io ports (Interactive)."},
	{ "puts",       do_puts,        "PUTS cmd - writes a command (cmd) to the PC-58." },
   { "Quit",       do_quit,        "QUIT - quits the application." },
	{ "sleep",      do_sleep,       "SLEEP sec - pauses the program of sec seconds." },
   { "Test",       do_test,        "TEST - a test command." },
	{ "WaitDone",   do_waitdone,    "WAITDONE mask - waits until done register equals mask." },
};
 
int num_parse_item = { sizeof(parse_table) / sizeof(struct PARSE_TABLE) };

int Quit,
    Fd;

/*---------------------------------
** main()
**---------------------------------
*/
int main( int argc, char * argv[])
{
	char buf[80];

   /*----------------------
   **  open device driver & Read config.
   */
   if( (Fd = open("/dev/pc580", O_RDWR, 0x666)) < 0)
   {
      printf("Error opening pc580 device driver \n");
      exit(0);
   }

   /* main loop for commands */
   Quit = FALSE;
   while( !Quit )
   {
      /* Get command */
      printf("Enter Command: ");
      gets( buf );
 
      cmd_execute( buf );
   }

   close( Fd );

	return 0;
}

/*-------------------------------------------------------------
** cmd_execute() - Executes a command line instruction
**-------------------------------------------------------------
*/
int cmd_execute( cmd_buf )
   char * cmd_buf;
{
   char copy_buf[80];
   int error,
      found,
      i;
 
   char * kw,
        * par;
   /*
   **  Search table for keyword
   */
   strxcpy( copy_buf, cmd_buf, sizeof(copy_buf));
 
   kw  = strtok(copy_buf, " ");
   if( (kw == NULL) || (*kw=='#') )
      return( ERR_NONE );

   if( (par = strtok( (char*)NULL, "")) != NULL )
		unpad( par, ' ' );

   /*
   **  Search table for keyword. If found call function.
   */
   error = ERR_INV_KW;
   for(i=0, found = FALSE; i<num_parse_item && found == FALSE; )
   {
      if( !stricmp( kw, parse_table[i].kw ) )
      {
         found = TRUE;
         error = (parse_table[i].pfi)(par);
      }
      else
         i++;
   }
   /*
   **  Provide user feedback
   */
 
   if( error != ERR_NONE )
   {
      printf("%s - %s\n", cmd_buf, Error_string[abs(error)]);
   }
 
   return( error );
}

/*--------------------------------------------------------------
**  cmd_search_selection_list() - search a list for item.
**     Return:  >= 0         =  the index of the matching item.
**              ERR_INV_RNG  =  Item is not in list.
**--------------------------------------------------------------
*/
int cmd_search_selection_list( item, list)
   char * item;    /* Item to locate in list            */
   char ** list;   /* This list must be NULL terminaled */
{
   int i;
 
   if( item == NULL )
      return ERR_INV_RNG;
 
   for( i=0; list[i] != NULL; i++)
   {
     if( !stricmp( item, list[i] ))
       return i;
   }
   return ERR_INV_RNG;
}
 
/********************************************************************
**                      COMMAND FUNCTIONS                          **
********************************************************************/

/*----------------------------------------------------------------------
**  do_axisinfo() - .
**----------------------------------------------------------------------
*/
int do_axisinfo( par )
   char * par;
{
	static int inx = 0;
	int rc;
	struct pc58_dram_axis_t axis;

   inx  = get_long(  " Inx ", inx,   0, 7);

   if( (rc = pc58_readAxisInfo( Fd, inx, &axis)) != ERR_NONE)
		return rc;
 
   printf("  encoder_pos=%ld\n",   axis.encoder_pos);
   printf("  command_pos=%ld\n",   axis.command_pos);
   printf("  command_vel=%ld\n",   axis.command_vel);
   printf("        accel=%ld\n",   axis.accel);
   printf("      max_vel=%ld\n",   axis.max_vel);
   printf("     base_vel=%ld\n",   axis.base_vel);
   printf("  filter_gain=%f\n",    axis.filter_gain);
   printf("  filter_pole=%ld\n",   axis.filter_pole);
   printf("  filter_zero=%ld\n",   axis.filter_zero);
   return ERR_NONE;

}

/*----------------------------------------------------------------------
**  do_cleardone() - clears the done flag 
**----------------------------------------------------------------------
*/
int do_cleardone( par )
   char * par;
{
	unsigned long mask;
 
   if( par == NULL)
      return ERR_INV_FORMAT;
 
   sscanf( par, "%lx", &mask);
   mask &= 0x00ff;
 
   if( !INRANGE(1, mask, 0xff))
      return ERR_INV_RNG;

   pc58_clearDoneFlag( Fd, mask);
   return ERR_NONE;
}

/*----------------------------------------------------------------------
**  do_beep() - output the beep character
**----------------------------------------------------------------------
*/
int do_beep( par )
   char * par;
{
   printf("\007\n");
	return ERR_NONE;
}

/*----------------------------------------------------------------------
**  do_do() - execute a file.
**----------------------------------------------------------------------
*/
int do_do( par )
   char * par;
{
   char filename[40];
   long reps;
   int  i;
   char *cptr;
   char buf[80];

   static FILE *fp = NULL;

   if( fp != NULL)
      return ERR_NOT_READY;

   if( (cptr = strtok( par, " ")) == NULL )
      return ERR_INV_FORMAT;
   strxcpy( filename, cptr, sizeof(filename));
   unpad( filename, ' ');

   if( -1 == my_atol( strtok(NULL, " "), &reps) )
      reps = 1;

   for( i=0; i<reps; i++)
   {
		int line;
      printf("Executing file %s. Reps %d of %ld\n", filename, i+1, reps+1);
      printf("========================================================\n");

      if( NULL == ( fp = fopen( filename, "r")) )
         return ERR_FILE_READ;

      line = 1;
	   while( fgets( buf, sizeof(buf), fp) != NULL)
      {
         str_rws( buf, sizeof(buf));
         unpad( buf, ' ');
			printf("%d. Command=[%s]\n", line++, buf);

         cmd_execute( buf );
      }
       fclose( fp );
      fp = NULL;
   }
   return ERR_NONE;
}

/*----------------------------------------------------------------------
**  do_getconfig() - read and display driver's configuration
**----------------------------------------------------------------------
*/
int do_getconfig( par )
   char * par;
{
   int rc;
   struct pc58_table_t cf;

	rc = pc58_getConfig( Fd, &cf );

	printf("    rc = %d\n", rc);
	printf("  base = 0x%x or %d\n", cf.base, cf.base);
	printf("  shmem = 0x%x \n", cf.shared_mem);
	printf(" status = 0x%x \n", cf.status_reg);
	printf("   slip = 0x%x \n", cf.slip_reg);
	printf("   done = 0x%x \n", cf.done_reg);
	printf("  limit = 0x%x \n", cf.limit_reg);
	printf("   home = 0x%x \n", cf.home_reg);

   return 0;
}

/*----------------------------------------------------------------------
**  do_gets() 
**----------------------------------------------------------------------
*/
int do_gets( par )
   char * par;
{
   int  rc;
   char buf[255];

   buf[0] = 0;
	rc = pc58_gets( Fd, buf, sizeof(buf) );
	printf("   pc58 outputs: %s \n", buf);

   return rc;
}

/*----------------------------------------------------------------------
**  do_help() 
**----------------------------------------------------------------------
*/
int do_help( par )
   char * par;
{
   int i;

   printf("\n"
			 "This program communicate with the PC-58 driver. It was written\n"
			 "as a test application for the pc58 device driver.\n" 
			 "Summary of commands:\n\n");

   for( i=0; i<num_parse_item; i++)
	{
		printf("   %s\n", parse_table[i].desc);
	}
	printf("\n");
   return ERR_NONE;
}

/*----------------------------------------------------------------------
**  do_io() 
**----------------------------------------------------------------------
*/
int do_io( par )
   char * par;
{
   int  rc;
   char buf[255];

   while( 1 )
	{
		printf("  Enter pc58 command > " ); fflush(stdout);
		fgets( buf, sizeof(buf), stdin );

       if( strlen(buf) <= 1)
			 return ERR_NONE;

		if( (rc = pc58_command( Fd, buf, buf, sizeof(buf) )) != ERR_NONE)
			printf("ERROR: pc58_command()=%d\n", rc);

		printf("   pc58 outputs: %s \n\n", buf);
   }
   return ERR_NONE;
}


/*----------------------------------------------------------------------
**  do_peek() 
**----------------------------------------------------------------------
*/
int do_peek( par )
   char * par;
{
   int data;
   static int port = 0x300;

	while( 1 ) 
	{
		port = get_ulong( "Port ", port, 0, 0x400);
		if( port == 0 )
			return ERR_NONE;


	   data = pc58_inb_p( Fd, port);
		printf("   port 0x%x = %d 0x%x \n\n", port, data, data);
	}

	return ERR_NONE;
}

/*----------------------------------------------------------------------
**  do_poke() 
**----------------------------------------------------------------------
*/
int do_poke( par )
   char * par;
{
   static int data = 0;
   static int port = 0x300;

	while( 1 ) 
	{
		port = get_ulong( "Port ", port, 0, 0x400);
		data = get_ulong( "Data ", data, 0, 0xff);
		if( port == 0 )
			return ERR_NONE;

	   pc58_outb_p( Fd, data, port);
	}

	return ERR_NONE;
}

/*----------------------------------------------------------------------
**  do_puts() - send a command to the pc58
**     Syntax: Puts  command
**----------------------------------------------------------------------
*/
int do_puts( par )
   char * par;
{
   int  rc;
   char buf[100];

   if( par == NULL )
      return ERR_NONE;

   strxcpy( buf, par, sizeof(buf)-5);
   strcat( buf, " \n");
	pc58_puts( Fd, buf );

   usleep( 200000 );

   buf[0] = 0;
	rc = pc58_gets( Fd, buf, sizeof(buf) );
	str_rws( buf, sizeof(buf));
	printf("   pc58 outputs: %s \n", buf);

   return rc;
}


/*----------------------------------------------------------------------
**  do_quit() - set Quit flag to TRUE.
**----------------------------------------------------------------------
*/
int do_quit( par )
   char * par;
{
   Quit = TRUE;
   return 0;
}

/*----------------------------------------------------------------------
**  do_sleep() -  Pauses for sec seconds.
**     Syntax: sleep sec
**----------------------------------------------------------------------
*/
int do_sleep( par )
   char * par;
{
   int i;
   long reps;
 
   if( -1 == my_atol( par, &reps) )
      return ERR_INV_FORMAT;
 
   printf("   Sleeping...");
   for( i=0; i< reps; i++ )
   {
      printf("z"); fflush(stdout);
      sleep(1);
   }
   printf("\n");
   return ERR_NONE;
}

/*----------------------------------------------------------------------
**  do_test() - .
**----------------------------------------------------------------------
*/
int do_test( par )
   char * par;
{
   return ERR_NONE;
}

/*----------------------------------------------------------------------
**  do_waitdone() - Waits until the done flag == mask
**     Syntax: wait mask
**----------------------------------------------------------------------
*/
int do_waitdone( par )
   char * par;
{
   int i, rc;
	unsigned long mask;
   struct pc58_table_t cf;

   if( par == NULL)
      return ERR_INV_FORMAT;
 
   sscanf( par, "%lx", &mask);
   mask &= 0x00ff;
 
   if( !INRANGE(1, mask, 0xff))
      return ERR_INV_RNG;
 
   rc = ERR_NONE;

   for( i=0; ; i++ )
   {
	   pc58_getConfig( Fd, &cf );

      printf("\r   WaitDone %d sec...0x%x == 0x%lx ", i, cf.done_reg, mask ); 
      fflush(stdout);

      if( (cf.done_reg & mask) == mask)
         break;

      sleep(1);
   }
   printf("\n");
   return ERR_NONE;
}


/********************************************************************
**                     SUPPORTING FUNCTIONS                        **
********************************************************************/


/*----------------------------------------------------------------------
**  get_ulong() - prompt the user and reads a ulong from the keyboard.
**----------------------------------------------------------------------
*/
ulong get_ulong (  char *prompt, ulong cur_val, ulong min, ulong max )
{
   char buf[80];
 
   while( 1 )
   {
      printf( "%s (0x%lx): ", prompt, cur_val );
      gets( buf );
      unpad( buf, ' ' );
      if( strlen( buf) >= 1 )
         sscanf( buf, "%lx", &cur_val);
 
      if( INRANGE(min, cur_val, max))
      {
         return cur_val;
      }
      else
         printf("\007INVALID Input. Range is %lu to %lu.\n", min, max);
   }
}
 
/*----------------------------------------------------------------------
**  get_long() - prompt the user and reads a long from the keyboard.
**----------------------------------------------------------------------
*/
long get_long (  char *prompt, long cur_val, long min, long max )
{
   char buf[80];
 
   while( 1 )
   {
      printf( "%s (%ld): ", prompt, cur_val );
      gets( buf );
      unpad( buf, ' ' );
      my_atol(buf, &cur_val);
 
      if( INRANGE(min, cur_val, max))
      {
         return cur_val;
      }
      else
         printf("\007INVALID Input. Range is %ld to %ld.\n", min, max);
   }
}
 
/*----------------------------------------------------------------------
**  get_dbl() - prompt the user and reads a double from the keyboard.
**----------------------------------------------------------------------
*/
double get_dbl( char *prompt, double cur_val, double min, double max )
{
   char buf[80];
 
   while( 1 )
   {
      printf( "%s (%f): ", prompt, cur_val );
      gets( buf );
      unpad( buf, ' ' );
      if( strlen( buf) >= 1 )
         sscanf( buf, "%lf", &cur_val);
      if( INRANGE( min, cur_val, max))
         return cur_val;
      else
         printf("\007INVALID Input. Range is %g to %g.\n", min, max);
   }
}


